# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AggregationGranularity(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of AggregationGranularity."""

    DAILY = "Daily"
    """Daily."""
    HOURLY = "Hourly"
    """Hourly."""


class OfferTermInfoName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Name of the offer term."""

    RECURRING_CHARGE = "Recurring Charge"
    """Recurring Charge."""
    MONETARY_COMMITMENT = "Monetary Commitment"
    """Monetary Commitment."""
    MONETARY_CREDIT = "Monetary Credit"
    """Monetary Credit."""
