# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import IndividuallyParentClientTestBase, IndividuallyParentPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestIndividuallyParentIndividuallyParentNestedWithMixedClientOperations(IndividuallyParentClientTestBase):
    @IndividuallyParentPreparer()
    @recorded_by_proxy
    def test_individually_parent_nested_with_mixed_client_with_query(self, individuallyparent_endpoint):
        client = self.create_client(endpoint=individuallyparent_endpoint)
        response = client.individually_parent_nested_with_mixed_client.with_query(
            region="str",
        )

        # please add some check logic here by yourself
        # ...

    @IndividuallyParentPreparer()
    @recorded_by_proxy
    def test_individually_parent_nested_with_mixed_client_get_standalone(self, individuallyparent_endpoint):
        client = self.create_client(endpoint=individuallyparent_endpoint)
        response = client.individually_parent_nested_with_mixed_client.get_standalone(
            region="str",
        )

        # please add some check logic here by yourself
        # ...

    @IndividuallyParentPreparer()
    @recorded_by_proxy
    def test_individually_parent_nested_with_mixed_client_delete_standalone(self, individuallyparent_endpoint):
        client = self.create_client(endpoint=individuallyparent_endpoint)
        response = client.individually_parent_nested_with_mixed_client.delete_standalone(
            region="str",
        )

        # please add some check logic here by yourself
        # ...
