# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import DocumentationPreparer
from testpreparer_async import DocumentationClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDocumentationListsOperationsAsync(DocumentationClientTestBaseAsync):
    @DocumentationPreparer()
    @recorded_by_proxy_async
    async def test_lists_bullet_points_op(self, documentation_endpoint):
        client = self.create_async_client(endpoint=documentation_endpoint)
        response = await client.lists.bullet_points_op()

        # please add some check logic here by yourself
        # ...

    @DocumentationPreparer()
    @recorded_by_proxy_async
    async def test_lists_bullet_points_model(self, documentation_endpoint):
        client = self.create_async_client(endpoint=documentation_endpoint)
        response = await client.lists.bullet_points_model(
            body={"input": {"prop": "str"}},
            input={"prop": "str"},
        )

        # please add some check logic here by yourself
        # ...

    @DocumentationPreparer()
    @recorded_by_proxy_async
    async def test_lists_numbered(self, documentation_endpoint):
        client = self.create_async_client(endpoint=documentation_endpoint)
        response = await client.lists.numbered()

        # please add some check logic here by yourself
        # ...
