# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import DurationClientTestBase, DurationPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDurationHeaderOperations(DurationClientTestBase):
    @DurationPreparer()
    @recorded_by_proxy
    def test_header_default(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.default(
            duration="1 day, 0:00:00",
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_iso8601(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.iso8601(
            duration="1 day, 0:00:00",
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_iso8601_array(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.iso8601_array(
            duration=["1 day, 0:00:00"],
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_int32_seconds(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.int32_seconds(
            duration=0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_int32_seconds_larger_unit(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.int32_seconds_larger_unit(
            duration=0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_float_seconds(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.float_seconds(
            duration=0.0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_float_seconds_larger_unit(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.float_seconds_larger_unit(
            duration=0.0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_float64_seconds(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.float64_seconds(
            duration=0.0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_int32_milliseconds(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.int32_milliseconds(
            duration=0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_int32_milliseconds_larger_unit(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.int32_milliseconds_larger_unit(
            duration=0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_float_milliseconds(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.float_milliseconds(
            duration=0.0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_float_milliseconds_larger_unit(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.float_milliseconds_larger_unit(
            duration=0.0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_float64_milliseconds(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.float64_milliseconds(
            duration=0.0,
        )

        # please add some check logic here by yourself
        # ...

    @DurationPreparer()
    @recorded_by_proxy
    def test_header_int32_milliseconds_array(self, duration_endpoint):
        client = self.create_client(endpoint=duration_endpoint)
        response = client.header.int32_milliseconds_array(
            duration=[0],
        )

        # please add some check logic here by yourself
        # ...
