# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import SimpleModelValueOperations  # type: ignore
from ._operations import ModelWithRenamedPropertyValueOperations  # type: ignore
from ._operations import ModelWithRenamedFieldsValueOperations  # type: ignore
from ._operations import ModelWithNestedModelValueOperations  # type: ignore
from ._operations import ModelWithRenamedNestedModelValueOperations  # type: ignore
from ._operations import ModelWithSimpleArraysValueOperations  # type: ignore
from ._operations import ModelWithUnwrappedArrayValueOperations  # type: ignore
from ._operations import ModelWithRenamedArraysValueOperations  # type: ignore
from ._operations import ModelWithWrappedPrimitiveCustomItemNamesValueOperations  # type: ignore
from ._operations import ModelWithArrayOfModelValueOperations  # type: ignore
from ._operations import ModelWithUnwrappedModelArrayValueOperations  # type: ignore
from ._operations import ModelWithRenamedWrappedModelArrayValueOperations  # type: ignore
from ._operations import ModelWithRenamedUnwrappedModelArrayValueOperations  # type: ignore
from ._operations import ModelWithRenamedWrappedAndItemModelArrayValueOperations  # type: ignore
from ._operations import ModelWithAttributesValueOperations  # type: ignore
from ._operations import ModelWithRenamedAttributeValueOperations  # type: ignore
from ._operations import ModelWithNamespaceValueOperations  # type: ignore
from ._operations import ModelWithNamespaceOnPropertiesValueOperations  # type: ignore
from ._operations import ModelWithTextValueOperations  # type: ignore
from ._operations import ModelWithOptionalFieldValueOperations  # type: ignore
from ._operations import ModelWithEmptyArrayValueOperations  # type: ignore
from ._operations import ModelWithDictionaryValueOperations  # type: ignore
from ._operations import ModelWithEncodedNamesValueOperations  # type: ignore
from ._operations import ModelWithEnumValueOperations  # type: ignore
from ._operations import ModelWithDatetimeValueOperations  # type: ignore
from ._operations import XmlErrorValueOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "SimpleModelValueOperations",
    "ModelWithRenamedPropertyValueOperations",
    "ModelWithRenamedFieldsValueOperations",
    "ModelWithNestedModelValueOperations",
    "ModelWithRenamedNestedModelValueOperations",
    "ModelWithSimpleArraysValueOperations",
    "ModelWithUnwrappedArrayValueOperations",
    "ModelWithRenamedArraysValueOperations",
    "ModelWithWrappedPrimitiveCustomItemNamesValueOperations",
    "ModelWithArrayOfModelValueOperations",
    "ModelWithUnwrappedModelArrayValueOperations",
    "ModelWithRenamedWrappedModelArrayValueOperations",
    "ModelWithRenamedUnwrappedModelArrayValueOperations",
    "ModelWithRenamedWrappedAndItemModelArrayValueOperations",
    "ModelWithAttributesValueOperations",
    "ModelWithRenamedAttributeValueOperations",
    "ModelWithNamespaceValueOperations",
    "ModelWithNamespaceOnPropertiesValueOperations",
    "ModelWithTextValueOperations",
    "ModelWithOptionalFieldValueOperations",
    "ModelWithEmptyArrayValueOperations",
    "ModelWithDictionaryValueOperations",
    "ModelWithEncodedNamesValueOperations",
    "ModelWithEnumValueOperations",
    "ModelWithDatetimeValueOperations",
    "XmlErrorValueOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
