# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appnetwork import AppNetworkMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appnetwork
# USAGE
    python app_link_members_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AppNetworkMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.app_link_members.begin_create_or_update(
        resource_group_name="test_rg",
        app_link_name="applink-test-01",
        app_link_member_name="member-01",
        resource={
            "location": "westus2",
            "properties": {
                "clusterType": "AKS",
                "connectivityProfile": {
                    "eastWestGateway": {"visibility": "Internal"},
                    "privateConnect": {
                        "subnetResourceId": "/subscriptions/bc7e0da9-5e4c-4a91-9252-9658837006cf/resourceGroups/applink-vnet-rg/providers/Microsoft.Network/virtualNetworks/vnet1/subnets/subnet1"
                    },
                },
                "metadata": {
                    "resourceId": "/subscriptions/bc7e0da9-5e4c-4a91-9252-9658837006cf/resourcegroups/applink-rg/providers/Microsoft.ContainerService/managedClusters/applink-member1"
                },
                "upgradeProfile": {"fullyManagedUpgradeProfile": {"releaseChannel": "Stable"}, "mode": "FullyManaged"},
            },
            "tags": {"key2913": "test_tag"},
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-08-01-preview/AppLinkMembers_CreateOrUpdate.json
if __name__ == "__main__":
    main()
