# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.authorization.aio import AuthorizationManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAuthorizationManagementAccessReviewDefaultSettingsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AuthorizationManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_access_review_default_settings_get(self, resource_group):
        response = await self.client.access_review_default_settings.get(
            api_version="2021-12-01-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_access_review_default_settings_put(self, resource_group):
        response = await self.client.access_review_default_settings.put(
            properties={
                "autoApplyDecisionsEnabled": bool,
                "defaultDecision": "str",
                "defaultDecisionEnabled": bool,
                "instanceDurationInDays": 0,
                "justificationRequiredOnApproval": bool,
                "mailNotificationsEnabled": bool,
                "recommendationLookBackDuration": "1 day, 0:00:00",
                "recommendationsEnabled": bool,
                "recurrence": {
                    "pattern": {"interval": 0, "type": "str"},
                    "range": {
                        "endDate": "2020-02-20 00:00:00",
                        "numberOfOccurrences": 0,
                        "startDate": "2020-02-20 00:00:00",
                        "type": "str",
                    },
                },
                "reminderNotificationsEnabled": bool,
            },
            api_version="2021-12-01-preview",
        )

        # please add some check logic here by yourself
        # ...
