# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class EnumV2(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of EnumV2."""

    ENUM_MEMBER_V2 = "enumMemberV2"
    """ENUM_MEMBER_V2."""


class EnumV3(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of EnumV3."""

    ENUM_MEMBER_V1 = "enumMemberV1"
    """ENUM_MEMBER_V1."""
    ENUM_MEMBER_V2_PREVIEW = "enumMemberV2Preview"
    """ENUM_MEMBER_V2_PREVIEW."""


class Versions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The version of the API."""

    V1 = "v1"
    """The version v1."""
    V2_PREVIEW = "v2preview"
    """The V2 Preview version."""
    V2 = "v2"
    """The version v2."""
