# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import DocumentationClientTestBase, DocumentationPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDocumentationTextFormattingOperations(DocumentationClientTestBase):
    @DocumentationPreparer()
    @recorded_by_proxy
    def test_text_formatting_bold_text(self, documentation_endpoint):
        client = self.create_client(endpoint=documentation_endpoint)
        response = client.text_formatting.bold_text()

        # please add some check logic here by yourself
        # ...

    @DocumentationPreparer()
    @recorded_by_proxy
    def test_text_formatting_italic_text(self, documentation_endpoint):
        client = self.create_client(endpoint=documentation_endpoint)
        response = client.text_formatting.italic_text()

        # please add some check logic here by yourself
        # ...

    @DocumentationPreparer()
    @recorded_by_proxy
    def test_text_formatting_combined_formatting(self, documentation_endpoint):
        client = self.create_client(endpoint=documentation_endpoint)
        response = client.text_formatting.combined_formatting()

        # please add some check logic here by yourself
        # ...
