# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import AttestationClientTestBase, AttestationPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestAttestationPolicyCertificatesOperations(AttestationClientTestBase):
    @AttestationPreparer()
    @recorded_by_proxy
    def test_policy_certificates_get(self, attestation_endpoint):
        client = self.create_client(endpoint=attestation_endpoint)
        response = client.policy_certificates.get()

        # please add some check logic here by yourself
        # ...

    @AttestationPreparer()
    @recorded_by_proxy
    def test_policy_certificates_add(self, attestation_endpoint):
        client = self.create_client(endpoint=attestation_endpoint)
        response = client.policy_certificates.add(
            body={"certificate": "str"},
            certificate="str",
        )

        # please add some check logic here by yourself
        # ...

    @AttestationPreparer()
    @recorded_by_proxy
    def test_policy_certificates_remove(self, attestation_endpoint):
        client = self.create_client(endpoint=attestation_endpoint)
        response = client.policy_certificates.remove(
            body={"certificate": "str"},
            certificate="str",
        )

        # please add some check logic here by yourself
        # ...
