# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import IndividuallyParentPreparer
from testpreparer_async import IndividuallyParentClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestIndividuallyParentIndividuallyParentNestedWithPathClientOperationsAsync(
    IndividuallyParentClientTestBaseAsync
):
    @IndividuallyParentPreparer()
    @recorded_by_proxy_async
    async def test_individually_parent_nested_with_path_client_with_query(self, individuallyparent_endpoint):
        client = self.create_async_client(endpoint=individuallyparent_endpoint)
        response = await client.individually_parent_nested_with_path_client.with_query()

        # please add some check logic here by yourself
        # ...

    @IndividuallyParentPreparer()
    @recorded_by_proxy_async
    async def test_individually_parent_nested_with_path_client_get_standalone(self, individuallyparent_endpoint):
        client = self.create_async_client(endpoint=individuallyparent_endpoint)
        response = await client.individually_parent_nested_with_path_client.get_standalone()

        # please add some check logic here by yourself
        # ...

    @IndividuallyParentPreparer()
    @recorded_by_proxy_async
    async def test_individually_parent_nested_with_path_client_delete_standalone(self, individuallyparent_endpoint):
        client = self.create_async_client(endpoint=individuallyparent_endpoint)
        response = await client.individually_parent_nested_with_path_client.delete_standalone()

        # please add some check logic here by yourself
        # ...
