# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from client.structure.clientoperationgroup.aio import FirstClient, SecondClient
from devtools_testutils import AzureRecordedTestCase


class FirstClientTestBaseAsync(AzureRecordedTestCase):

    def create_async_client(self, endpoint):
        credential = self.get_credential(FirstClient, is_async=True)
        return self.create_client_from_credential(
            FirstClient,
            credential=credential,
            endpoint=endpoint,
        )


class SecondClientTestBaseAsync(AzureRecordedTestCase):

    def create_async_client(self, endpoint):
        credential = self.get_credential(SecondClient, is_async=True)
        return self.create_client_from_credential(
            SecondClient,
            credential=credential,
            endpoint=endpoint,
        )
