# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.billingbenefits import BillingBenefitsMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestBillingBenefitsMgmtFreeServicesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(BillingBenefitsMgmtClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_free_services_get(self, resource_group):
        response = self.client.free_services.get(
            resource_group_name=resource_group.name,
            free_service_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_free_services_begin_create(self, resource_group):
        response = self.client.free_services.begin_create(
            resource_group_name=resource_group.name,
            free_service_name="str",
            body={
                "location": "str",
                "etag": "str",
                "id": "str",
                "identity": {
                    "type": "str",
                    "principalId": "str",
                    "tenantId": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "kind": "str",
                "managedBy": "str",
                "name": "str",
                "plan": {"name": "str", "product": "str", "publisher": "str", "promotionCode": "str", "version": "str"},
                "properties": {
                    "billingAccountResourceId": "str",
                    "billingProfileResourceId": "str",
                    "customerResourceId": "str",
                    "endAt": "2020-02-20 00:00:00",
                    "productCode": "str",
                    "provisioningState": "str",
                    "startAt": "2020-02-20 00:00:00",
                    "status": "str",
                    "systemId": "str",
                },
                "sku": {"name": "str", "capacity": 0, "family": "str", "size": "str", "tier": "str"},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_free_services_begin_update(self, resource_group):
        response = self.client.free_services.begin_update(
            resource_group_name=resource_group.name,
            free_service_name="str",
            body={"properties": {"endAt": "2020-02-20 00:00:00"}, "tags": {"str": "str"}},
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_free_services_begin_delete(self, resource_group):
        response = self.client.free_services.begin_delete(
            resource_group_name=resource_group.name,
            free_service_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_free_services_list_by_resource_group(self, resource_group):
        response = self.client.free_services.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_free_services_list_by_subscription(self, resource_group):
        response = self.client.free_services.list_by_subscription()
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...
