# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.cognitiveservices.aio import CognitiveServicesManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCognitiveServicesManagementRaiBlocklistItemsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CognitiveServicesManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_get(self, resource_group):
        response = await self.client.rai_blocklist_items.get(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
            rai_blocklist_item_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_create_or_update(self, resource_group):
        response = await self.client.rai_blocklist_items.create_or_update(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
            rai_blocklist_item_name="str",
            rai_blocklist_item={
                "etag": "str",
                "id": "str",
                "name": "str",
                "properties": {"isRegex": bool, "pattern": "str"},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_begin_delete(self, resource_group):
        response = await (
            await self.client.rai_blocklist_items.begin_delete(
                resource_group_name=resource_group.name,
                account_name="str",
                rai_blocklist_name="str",
                rai_blocklist_item_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_list(self, resource_group):
        response = self.client.rai_blocklist_items.list(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_batch_add(self, resource_group):
        response = await self.client.rai_blocklist_items.batch_add(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
            rai_blocklist_items=[{"name": "str", "properties": {"isRegex": bool, "pattern": "str"}}],
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_rai_blocklist_items_batch_delete(self, resource_group):
        response = await self.client.rai_blocklist_items.batch_delete(
            resource_group_name=resource_group.name,
            account_name="str",
            rai_blocklist_name="str",
            rai_blocklist_items_names=["str"],
        )

        # please add some check logic here by yourself
        # ...
