# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import AFDDomain
from ._models_py3 import AFDDomainHttpsParameters
from ._models_py3 import AFDDomainListResult
from ._models_py3 import AFDDomainProperties
from ._models_py3 import AFDDomainUpdateParameters
from ._models_py3 import AFDDomainUpdatePropertiesParameters
from ._models_py3 import AFDEndpoint
from ._models_py3 import AFDEndpointListResult
from ._models_py3 import AFDEndpointProperties
from ._models_py3 import AFDEndpointPropertiesUpdateParameters
from ._models_py3 import AFDEndpointUpdateParameters
from ._models_py3 import AFDOrigin
from ._models_py3 import AFDOriginGroup
from ._models_py3 import AFDOriginGroupListResult
from ._models_py3 import AFDOriginGroupProperties
from ._models_py3 import AFDOriginGroupUpdateParameters
from ._models_py3 import AFDOriginGroupUpdatePropertiesParameters
from ._models_py3 import AFDOriginListResult
from ._models_py3 import AFDOriginProperties
from ._models_py3 import AFDOriginUpdateParameters
from ._models_py3 import AFDOriginUpdatePropertiesParameters
from ._models_py3 import AFDStateProperties
from ._models_py3 import ActivatedResourceReference
from ._models_py3 import AfdErrorResponse
from ._models_py3 import AfdPurgeParameters
from ._models_py3 import AfdRouteCacheConfiguration
from ._models_py3 import AzureFirstPartyManagedCertificate
from ._models_py3 import AzureFirstPartyManagedCertificateParameters
from ._models_py3 import CacheConfiguration
from ._models_py3 import CacheExpirationActionParameters
from ._models_py3 import CacheKeyQueryStringActionParameters
from ._models_py3 import CanMigrateParameters
from ._models_py3 import CanMigrateResult
from ._models_py3 import CdnCertificateSourceParameters
from ._models_py3 import CdnEndpoint
from ._models_py3 import CdnManagedHttpsParameters
from ._models_py3 import CdnWebApplicationFirewallPolicy
from ._models_py3 import CdnWebApplicationFirewallPolicyList
from ._models_py3 import CdnWebApplicationFirewallPolicyPatchParameters
from ._models_py3 import Certificate
from ._models_py3 import CheckEndpointNameAvailabilityInput
from ._models_py3 import CheckEndpointNameAvailabilityOutput
from ._models_py3 import CheckHostNameAvailabilityInput
from ._models_py3 import CheckNameAvailabilityInput
from ._models_py3 import CheckNameAvailabilityOutput
from ._models_py3 import CidrIpAddress
from ._models_py3 import ClientPortMatchConditionParameters
from ._models_py3 import Components18OrqelSchemasWafmetricsresponsePropertiesSeriesItemsPropertiesDataItems
from ._models_py3 import Components1Gs0LlpSchemasMetricsresponsePropertiesSeriesItemsPropertiesDataItems
from ._models_py3 import ComponentsKpo1PjSchemasWafrankingsresponsePropertiesDataItemsPropertiesMetricsItems
from ._models_py3 import CompressionSettings
from ._models_py3 import ContinentsResponse
from ._models_py3 import ContinentsResponseContinentsItem
from ._models_py3 import ContinentsResponseCountryOrRegionsItem
from ._models_py3 import CookiesMatchConditionParameters
from ._models_py3 import CustomDomain
from ._models_py3 import CustomDomainHttpsParameters
from ._models_py3 import CustomDomainListResult
from ._models_py3 import CustomDomainParameters
from ._models_py3 import CustomRule
from ._models_py3 import CustomRuleList
from ._models_py3 import CustomerCertificate
from ._models_py3 import CustomerCertificateParameters
from ._models_py3 import DeepCreatedCustomDomain
from ._models_py3 import DeepCreatedOrigin
from ._models_py3 import DeepCreatedOriginGroup
from ._models_py3 import DeliveryRule
from ._models_py3 import DeliveryRuleAction
from ._models_py3 import DeliveryRuleCacheExpirationAction
from ._models_py3 import DeliveryRuleCacheKeyQueryStringAction
from ._models_py3 import DeliveryRuleClientPortCondition
from ._models_py3 import DeliveryRuleCondition
from ._models_py3 import DeliveryRuleCookiesCondition
from ._models_py3 import DeliveryRuleHostNameCondition
from ._models_py3 import DeliveryRuleHttpVersionCondition
from ._models_py3 import DeliveryRuleIsDeviceCondition
from ._models_py3 import DeliveryRulePostArgsCondition
from ._models_py3 import DeliveryRuleQueryStringCondition
from ._models_py3 import DeliveryRuleRemoteAddressCondition
from ._models_py3 import DeliveryRuleRequestBodyCondition
from ._models_py3 import DeliveryRuleRequestHeaderAction
from ._models_py3 import DeliveryRuleRequestHeaderCondition
from ._models_py3 import DeliveryRuleRequestMethodCondition
from ._models_py3 import DeliveryRuleRequestSchemeCondition
from ._models_py3 import DeliveryRuleRequestUriCondition
from ._models_py3 import DeliveryRuleResponseHeaderAction
from ._models_py3 import DeliveryRuleRouteConfigurationOverrideAction
from ._models_py3 import DeliveryRuleServerPortCondition
from ._models_py3 import DeliveryRuleSocketAddrCondition
from ._models_py3 import DeliveryRuleSslProtocolCondition
from ._models_py3 import DeliveryRuleUrlFileExtensionCondition
from ._models_py3 import DeliveryRuleUrlFileNameCondition
from ._models_py3 import DeliveryRuleUrlPathCondition
from ._models_py3 import DimensionProperties
from ._models_py3 import DomainValidationProperties
from ._models_py3 import EdgeNode
from ._models_py3 import EdgenodeResult
from ._models_py3 import Endpoint
from ._models_py3 import EndpointListResult
from ._models_py3 import EndpointProperties
from ._models_py3 import EndpointPropertiesUpdateParameters
from ._models_py3 import EndpointPropertiesUpdateParametersDeliveryPolicy
from ._models_py3 import EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink
from ._models_py3 import EndpointUpdateParameters
from ._models_py3 import ErrorAdditionalInfo
from ._models_py3 import ErrorDetail
from ._models_py3 import ErrorResponse
from ._models_py3 import GeoFilter
from ._models_py3 import HeaderActionParameters
from ._models_py3 import HealthProbeParameters
from ._models_py3 import HostNameMatchConditionParameters
from ._models_py3 import HttpErrorRangeParameters
from ._models_py3 import HttpVersionMatchConditionParameters
from ._models_py3 import IpAddressGroup
from ._models_py3 import IsDeviceMatchConditionParameters
from ._models_py3 import KeyVaultCertificateSourceParameters
from ._models_py3 import KeyVaultSigningKeyParameters
from ._models_py3 import LoadBalancingSettingsParameters
from ._models_py3 import LoadParameters
from ._models_py3 import LogSpecification
from ._models_py3 import ManagedCertificate
from ._models_py3 import ManagedCertificateParameters
from ._models_py3 import ManagedRuleDefinition
from ._models_py3 import ManagedRuleGroupDefinition
from ._models_py3 import ManagedRuleGroupOverride
from ._models_py3 import ManagedRuleOverride
from ._models_py3 import ManagedRuleSet
from ._models_py3 import ManagedRuleSetDefinition
from ._models_py3 import ManagedRuleSetDefinitionList
from ._models_py3 import ManagedRuleSetList
from ._models_py3 import ManagedServiceIdentity
from ._models_py3 import MatchCondition
from ._models_py3 import MetricAvailability
from ._models_py3 import MetricSpecification
from ._models_py3 import MetricsResponse
from ._models_py3 import MetricsResponseSeriesItem
from ._models_py3 import MetricsResponseSeriesPropertiesItemsItem
from ._models_py3 import MigrateResult
from ._models_py3 import MigrationErrorType
from ._models_py3 import MigrationParameters
from ._models_py3 import MigrationWebApplicationFirewallMapping
from ._models_py3 import Operation
from ._models_py3 import OperationDisplay
from ._models_py3 import OperationsListResult
from ._models_py3 import Origin
from ._models_py3 import OriginGroup
from ._models_py3 import OriginGroupListResult
from ._models_py3 import OriginGroupOverride
from ._models_py3 import OriginGroupOverrideAction
from ._models_py3 import OriginGroupOverrideActionParameters
from ._models_py3 import OriginGroupProperties
from ._models_py3 import OriginGroupUpdateParameters
from ._models_py3 import OriginGroupUpdatePropertiesParameters
from ._models_py3 import OriginListResult
from ._models_py3 import OriginProperties
from ._models_py3 import OriginUpdateParameters
from ._models_py3 import OriginUpdatePropertiesParameters
from ._models_py3 import PolicySettings
from ._models_py3 import PostArgsMatchConditionParameters
from ._models_py3 import Profile
from ._models_py3 import ProfileChangeSkuWafMapping
from ._models_py3 import ProfileListResult
from ._models_py3 import ProfileLogScrubbing
from ._models_py3 import ProfileScrubbingRules
from ._models_py3 import ProfileUpdateParameters
from ._models_py3 import ProfileUpgradeParameters
from ._models_py3 import ProxyResource
from ._models_py3 import PurgeParameters
from ._models_py3 import QueryStringMatchConditionParameters
from ._models_py3 import RankingsResponse
from ._models_py3 import RankingsResponseTablesItem
from ._models_py3 import RankingsResponseTablesPropertiesItemsItem
from ._models_py3 import RankingsResponseTablesPropertiesItemsMetricsItem
from ._models_py3 import RateLimitRule
from ._models_py3 import RateLimitRuleList
from ._models_py3 import RemoteAddressMatchConditionParameters
from ._models_py3 import RequestBodyMatchConditionParameters
from ._models_py3 import RequestHeaderMatchConditionParameters
from ._models_py3 import RequestMethodMatchConditionParameters
from ._models_py3 import RequestSchemeMatchConditionParameters
from ._models_py3 import RequestUriMatchConditionParameters
from ._models_py3 import Resource
from ._models_py3 import ResourceReference
from ._models_py3 import ResourceUsage
from ._models_py3 import ResourceUsageListResult
from ._models_py3 import ResourcesResponse
from ._models_py3 import ResourcesResponseCustomDomainsItem
from ._models_py3 import ResourcesResponseEndpointsItem
from ._models_py3 import ResourcesResponseEndpointsPropertiesItemsItem
from ._models_py3 import ResponseBasedOriginErrorDetectionParameters
from ._models_py3 import Route
from ._models_py3 import RouteConfigurationOverrideActionParameters
from ._models_py3 import RouteListResult
from ._models_py3 import RouteProperties
from ._models_py3 import RouteUpdateParameters
from ._models_py3 import RouteUpdatePropertiesParameters
from ._models_py3 import Rule
from ._models_py3 import RuleListResult
from ._models_py3 import RuleProperties
from ._models_py3 import RuleSet
from ._models_py3 import RuleSetListResult
from ._models_py3 import RuleSetProperties
from ._models_py3 import RuleUpdateParameters
from ._models_py3 import RuleUpdatePropertiesParameters
from ._models_py3 import Secret
from ._models_py3 import SecretListResult
from ._models_py3 import SecretParameters
from ._models_py3 import SecretProperties
from ._models_py3 import SecurityPolicy
from ._models_py3 import SecurityPolicyListResult
from ._models_py3 import SecurityPolicyProperties
from ._models_py3 import SecurityPolicyPropertiesParameters
from ._models_py3 import SecurityPolicyUpdateParameters
from ._models_py3 import SecurityPolicyWebApplicationFirewallAssociation
from ._models_py3 import SecurityPolicyWebApplicationFirewallParameters
from ._models_py3 import ServerPortMatchConditionParameters
from ._models_py3 import ServiceSpecification
from ._models_py3 import SharedPrivateLinkResourceProperties
from ._models_py3 import Sku
from ._models_py3 import SocketAddrMatchConditionParameters
from ._models_py3 import SslProtocolMatchConditionParameters
from ._models_py3 import SsoUri
from ._models_py3 import SupportedOptimizationTypesListResult
from ._models_py3 import SystemData
from ._models_py3 import TrackedResource
from ._models_py3 import UrlFileExtensionMatchConditionParameters
from ._models_py3 import UrlFileNameMatchConditionParameters
from ._models_py3 import UrlPathMatchConditionParameters
from ._models_py3 import UrlRedirectAction
from ._models_py3 import UrlRedirectActionParameters
from ._models_py3 import UrlRewriteAction
from ._models_py3 import UrlRewriteActionParameters
from ._models_py3 import UrlSigningAction
from ._models_py3 import UrlSigningActionParameters
from ._models_py3 import UrlSigningKey
from ._models_py3 import UrlSigningKeyParameters
from ._models_py3 import UrlSigningParamIdentifier
from ._models_py3 import Usage
from ._models_py3 import UsageName
from ._models_py3 import UsagesListResult
from ._models_py3 import UserAssignedIdentity
from ._models_py3 import UserManagedHttpsParameters
from ._models_py3 import ValidateCustomDomainInput
from ._models_py3 import ValidateCustomDomainOutput
from ._models_py3 import ValidateProbeInput
from ._models_py3 import ValidateProbeOutput
from ._models_py3 import ValidateSecretInput
from ._models_py3 import ValidateSecretOutput
from ._models_py3 import ValidationToken
from ._models_py3 import WafMetricsResponse
from ._models_py3 import WafMetricsResponseSeriesItem
from ._models_py3 import WafMetricsResponseSeriesPropertiesItemsItem
from ._models_py3 import WafRankingsResponse
from ._models_py3 import WafRankingsResponseDataItem

from ._cdn_management_client_enums import AFDEndpointProtocols
from ._cdn_management_client_enums import ActionType
from ._cdn_management_client_enums import AfdCertificateType
from ._cdn_management_client_enums import AfdMinimumTlsVersion
from ._cdn_management_client_enums import AfdProvisioningState
from ._cdn_management_client_enums import AfdQueryStringCachingBehavior
from ._cdn_management_client_enums import Algorithm
from ._cdn_management_client_enums import AutoGeneratedDomainNameLabelScope
from ._cdn_management_client_enums import CacheBehavior
from ._cdn_management_client_enums import CacheExpirationActionParametersTypeName
from ._cdn_management_client_enums import CacheKeyQueryStringActionParametersTypeName
from ._cdn_management_client_enums import CacheType
from ._cdn_management_client_enums import CanMigrateDefaultSku
from ._cdn_management_client_enums import CdnCertificateSourceParametersTypeName
from ._cdn_management_client_enums import CertificateSource
from ._cdn_management_client_enums import CertificateType
from ._cdn_management_client_enums import ClientPortMatchConditionParametersTypeName
from ._cdn_management_client_enums import ClientPortOperator
from ._cdn_management_client_enums import CookiesMatchConditionParametersTypeName
from ._cdn_management_client_enums import CookiesOperator
from ._cdn_management_client_enums import CustomDomainResourceState
from ._cdn_management_client_enums import CustomHttpsProvisioningState
from ._cdn_management_client_enums import CustomHttpsProvisioningSubstate
from ._cdn_management_client_enums import CustomRuleEnabledState
from ._cdn_management_client_enums import DeleteRule
from ._cdn_management_client_enums import DeliveryRuleActionEnum
from ._cdn_management_client_enums import DeploymentStatus
from ._cdn_management_client_enums import DestinationProtocol
from ._cdn_management_client_enums import DomainValidationState
from ._cdn_management_client_enums import EnabledState
from ._cdn_management_client_enums import EndpointProvisioningState
from ._cdn_management_client_enums import EndpointResourceState
from ._cdn_management_client_enums import ForwardingProtocol
from ._cdn_management_client_enums import GeoFilterActions
from ._cdn_management_client_enums import HeaderAction
from ._cdn_management_client_enums import HeaderActionParametersTypeName
from ._cdn_management_client_enums import HealthProbeRequestType
from ._cdn_management_client_enums import HostNameMatchConditionParametersTypeName
from ._cdn_management_client_enums import HostNameOperator
from ._cdn_management_client_enums import HttpVersionMatchConditionParametersTypeName
from ._cdn_management_client_enums import HttpVersionOperator
from ._cdn_management_client_enums import HttpsRedirect
from ._cdn_management_client_enums import IdentityType
from ._cdn_management_client_enums import IsDeviceMatchConditionParametersMatchValuesItem
from ._cdn_management_client_enums import IsDeviceMatchConditionParametersTypeName
from ._cdn_management_client_enums import IsDeviceOperator
from ._cdn_management_client_enums import KeyVaultCertificateSourceParametersTypeName
from ._cdn_management_client_enums import KeyVaultSigningKeyParametersTypeName
from ._cdn_management_client_enums import LinkToDefaultDomain
from ._cdn_management_client_enums import LogMetric
from ._cdn_management_client_enums import LogMetricsGranularity
from ._cdn_management_client_enums import LogMetricsGroupBy
from ._cdn_management_client_enums import LogRanking
from ._cdn_management_client_enums import LogRankingMetric
from ._cdn_management_client_enums import ManagedRuleEnabledState
from ._cdn_management_client_enums import ManagedServiceIdentityType
from ._cdn_management_client_enums import MatchProcessingBehavior
from ._cdn_management_client_enums import MatchVariable
from ._cdn_management_client_enums import MetricsGranularity
from ._cdn_management_client_enums import MetricsSeriesUnit
from ._cdn_management_client_enums import MinimumTlsVersion
from ._cdn_management_client_enums import Operator
from ._cdn_management_client_enums import OptimizationType
from ._cdn_management_client_enums import OriginGroupOverrideActionParametersTypeName
from ._cdn_management_client_enums import OriginGroupProvisioningState
from ._cdn_management_client_enums import OriginGroupResourceState
from ._cdn_management_client_enums import OriginProvisioningState
from ._cdn_management_client_enums import OriginResourceState
from ._cdn_management_client_enums import ParamIndicator
from ._cdn_management_client_enums import PolicyEnabledState
from ._cdn_management_client_enums import PolicyMode
from ._cdn_management_client_enums import PolicyResourceState
from ._cdn_management_client_enums import PolicySettingsDefaultCustomBlockResponseStatusCode
from ._cdn_management_client_enums import PostArgsMatchConditionParametersTypeName
from ._cdn_management_client_enums import PostArgsOperator
from ._cdn_management_client_enums import PrivateEndpointStatus
from ._cdn_management_client_enums import ProbeProtocol
from ._cdn_management_client_enums import ProfileProvisioningState
from ._cdn_management_client_enums import ProfileResourceState
from ._cdn_management_client_enums import ProfileScrubbingState
from ._cdn_management_client_enums import ProtocolType
from ._cdn_management_client_enums import ProvisioningState
from ._cdn_management_client_enums import QueryStringBehavior
from ._cdn_management_client_enums import QueryStringCachingBehavior
from ._cdn_management_client_enums import QueryStringMatchConditionParametersTypeName
from ._cdn_management_client_enums import QueryStringOperator
from ._cdn_management_client_enums import RedirectType
from ._cdn_management_client_enums import RemoteAddressMatchConditionParametersTypeName
from ._cdn_management_client_enums import RemoteAddressOperator
from ._cdn_management_client_enums import RequestBodyMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestBodyOperator
from ._cdn_management_client_enums import RequestHeaderMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestHeaderOperator
from ._cdn_management_client_enums import RequestMethodMatchConditionParametersMatchValuesItem
from ._cdn_management_client_enums import RequestMethodMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestMethodOperator
from ._cdn_management_client_enums import RequestSchemeMatchConditionParametersMatchValuesItem
from ._cdn_management_client_enums import RequestSchemeMatchConditionParametersOperator
from ._cdn_management_client_enums import RequestSchemeMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestUriMatchConditionParametersTypeName
from ._cdn_management_client_enums import RequestUriOperator
from ._cdn_management_client_enums import ResourceType
from ._cdn_management_client_enums import ResourceUsageUnit
from ._cdn_management_client_enums import ResponseBasedDetectedErrorTypes
from ._cdn_management_client_enums import RouteConfigurationOverrideActionParametersTypeName
from ._cdn_management_client_enums import RuleCacheBehavior
from ._cdn_management_client_enums import RuleIsCompressionEnabled
from ._cdn_management_client_enums import RuleQueryStringCachingBehavior
from ._cdn_management_client_enums import ScrubbingRuleEntryMatchOperator
from ._cdn_management_client_enums import ScrubbingRuleEntryMatchVariable
from ._cdn_management_client_enums import ScrubbingRuleEntryState
from ._cdn_management_client_enums import SecretType
from ._cdn_management_client_enums import SecurityPolicyType
from ._cdn_management_client_enums import ServerPortMatchConditionParametersTypeName
from ._cdn_management_client_enums import ServerPortOperator
from ._cdn_management_client_enums import SharedPrivateLinkResourceStatus
from ._cdn_management_client_enums import SkuName
from ._cdn_management_client_enums import SocketAddrMatchConditionParametersTypeName
from ._cdn_management_client_enums import SocketAddrOperator
from ._cdn_management_client_enums import SslProtocol
from ._cdn_management_client_enums import SslProtocolMatchConditionParametersTypeName
from ._cdn_management_client_enums import SslProtocolOperator
from ._cdn_management_client_enums import Status
from ._cdn_management_client_enums import Transform
from ._cdn_management_client_enums import TransformType
from ._cdn_management_client_enums import UpdateRule
from ._cdn_management_client_enums import UrlFileExtensionMatchConditionParametersTypeName
from ._cdn_management_client_enums import UrlFileExtensionOperator
from ._cdn_management_client_enums import UrlFileNameMatchConditionParametersTypeName
from ._cdn_management_client_enums import UrlFileNameOperator
from ._cdn_management_client_enums import UrlPathMatchConditionParametersTypeName
from ._cdn_management_client_enums import UrlPathOperator
from ._cdn_management_client_enums import UrlRedirectActionParametersTypeName
from ._cdn_management_client_enums import UrlRewriteActionParametersTypeName
from ._cdn_management_client_enums import UrlSigningActionParametersTypeName
from ._cdn_management_client_enums import UsageUnit
from ._cdn_management_client_enums import WafAction
from ._cdn_management_client_enums import WafGranularity
from ._cdn_management_client_enums import WafMatchVariable
from ._cdn_management_client_enums import WafMetric
from ._cdn_management_client_enums import WafMetricsGranularity
from ._cdn_management_client_enums import WafMetricsSeriesUnit
from ._cdn_management_client_enums import WafRankingGroupBy
from ._cdn_management_client_enums import WafRankingType
from ._cdn_management_client_enums import WafRuleType
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AFDDomain",
    "AFDDomainHttpsParameters",
    "AFDDomainListResult",
    "AFDDomainProperties",
    "AFDDomainUpdateParameters",
    "AFDDomainUpdatePropertiesParameters",
    "AFDEndpoint",
    "AFDEndpointListResult",
    "AFDEndpointProperties",
    "AFDEndpointPropertiesUpdateParameters",
    "AFDEndpointUpdateParameters",
    "AFDOrigin",
    "AFDOriginGroup",
    "AFDOriginGroupListResult",
    "AFDOriginGroupProperties",
    "AFDOriginGroupUpdateParameters",
    "AFDOriginGroupUpdatePropertiesParameters",
    "AFDOriginListResult",
    "AFDOriginProperties",
    "AFDOriginUpdateParameters",
    "AFDOriginUpdatePropertiesParameters",
    "AFDStateProperties",
    "ActivatedResourceReference",
    "AfdErrorResponse",
    "AfdPurgeParameters",
    "AfdRouteCacheConfiguration",
    "AzureFirstPartyManagedCertificate",
    "AzureFirstPartyManagedCertificateParameters",
    "CacheConfiguration",
    "CacheExpirationActionParameters",
    "CacheKeyQueryStringActionParameters",
    "CanMigrateParameters",
    "CanMigrateResult",
    "CdnCertificateSourceParameters",
    "CdnEndpoint",
    "CdnManagedHttpsParameters",
    "CdnWebApplicationFirewallPolicy",
    "CdnWebApplicationFirewallPolicyList",
    "CdnWebApplicationFirewallPolicyPatchParameters",
    "Certificate",
    "CheckEndpointNameAvailabilityInput",
    "CheckEndpointNameAvailabilityOutput",
    "CheckHostNameAvailabilityInput",
    "CheckNameAvailabilityInput",
    "CheckNameAvailabilityOutput",
    "CidrIpAddress",
    "ClientPortMatchConditionParameters",
    "Components18OrqelSchemasWafmetricsresponsePropertiesSeriesItemsPropertiesDataItems",
    "Components1Gs0LlpSchemasMetricsresponsePropertiesSeriesItemsPropertiesDataItems",
    "ComponentsKpo1PjSchemasWafrankingsresponsePropertiesDataItemsPropertiesMetricsItems",
    "CompressionSettings",
    "ContinentsResponse",
    "ContinentsResponseContinentsItem",
    "ContinentsResponseCountryOrRegionsItem",
    "CookiesMatchConditionParameters",
    "CustomDomain",
    "CustomDomainHttpsParameters",
    "CustomDomainListResult",
    "CustomDomainParameters",
    "CustomRule",
    "CustomRuleList",
    "CustomerCertificate",
    "CustomerCertificateParameters",
    "DeepCreatedCustomDomain",
    "DeepCreatedOrigin",
    "DeepCreatedOriginGroup",
    "DeliveryRule",
    "DeliveryRuleAction",
    "DeliveryRuleCacheExpirationAction",
    "DeliveryRuleCacheKeyQueryStringAction",
    "DeliveryRuleClientPortCondition",
    "DeliveryRuleCondition",
    "DeliveryRuleCookiesCondition",
    "DeliveryRuleHostNameCondition",
    "DeliveryRuleHttpVersionCondition",
    "DeliveryRuleIsDeviceCondition",
    "DeliveryRulePostArgsCondition",
    "DeliveryRuleQueryStringCondition",
    "DeliveryRuleRemoteAddressCondition",
    "DeliveryRuleRequestBodyCondition",
    "DeliveryRuleRequestHeaderAction",
    "DeliveryRuleRequestHeaderCondition",
    "DeliveryRuleRequestMethodCondition",
    "DeliveryRuleRequestSchemeCondition",
    "DeliveryRuleRequestUriCondition",
    "DeliveryRuleResponseHeaderAction",
    "DeliveryRuleRouteConfigurationOverrideAction",
    "DeliveryRuleServerPortCondition",
    "DeliveryRuleSocketAddrCondition",
    "DeliveryRuleSslProtocolCondition",
    "DeliveryRuleUrlFileExtensionCondition",
    "DeliveryRuleUrlFileNameCondition",
    "DeliveryRuleUrlPathCondition",
    "DimensionProperties",
    "DomainValidationProperties",
    "EdgeNode",
    "EdgenodeResult",
    "Endpoint",
    "EndpointListResult",
    "EndpointProperties",
    "EndpointPropertiesUpdateParameters",
    "EndpointPropertiesUpdateParametersDeliveryPolicy",
    "EndpointPropertiesUpdateParametersWebApplicationFirewallPolicyLink",
    "EndpointUpdateParameters",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "GeoFilter",
    "HeaderActionParameters",
    "HealthProbeParameters",
    "HostNameMatchConditionParameters",
    "HttpErrorRangeParameters",
    "HttpVersionMatchConditionParameters",
    "IpAddressGroup",
    "IsDeviceMatchConditionParameters",
    "KeyVaultCertificateSourceParameters",
    "KeyVaultSigningKeyParameters",
    "LoadBalancingSettingsParameters",
    "LoadParameters",
    "LogSpecification",
    "ManagedCertificate",
    "ManagedCertificateParameters",
    "ManagedRuleDefinition",
    "ManagedRuleGroupDefinition",
    "ManagedRuleGroupOverride",
    "ManagedRuleOverride",
    "ManagedRuleSet",
    "ManagedRuleSetDefinition",
    "ManagedRuleSetDefinitionList",
    "ManagedRuleSetList",
    "ManagedServiceIdentity",
    "MatchCondition",
    "MetricAvailability",
    "MetricSpecification",
    "MetricsResponse",
    "MetricsResponseSeriesItem",
    "MetricsResponseSeriesPropertiesItemsItem",
    "MigrateResult",
    "MigrationErrorType",
    "MigrationParameters",
    "MigrationWebApplicationFirewallMapping",
    "Operation",
    "OperationDisplay",
    "OperationsListResult",
    "Origin",
    "OriginGroup",
    "OriginGroupListResult",
    "OriginGroupOverride",
    "OriginGroupOverrideAction",
    "OriginGroupOverrideActionParameters",
    "OriginGroupProperties",
    "OriginGroupUpdateParameters",
    "OriginGroupUpdatePropertiesParameters",
    "OriginListResult",
    "OriginProperties",
    "OriginUpdateParameters",
    "OriginUpdatePropertiesParameters",
    "PolicySettings",
    "PostArgsMatchConditionParameters",
    "Profile",
    "ProfileChangeSkuWafMapping",
    "ProfileListResult",
    "ProfileLogScrubbing",
    "ProfileScrubbingRules",
    "ProfileUpdateParameters",
    "ProfileUpgradeParameters",
    "ProxyResource",
    "PurgeParameters",
    "QueryStringMatchConditionParameters",
    "RankingsResponse",
    "RankingsResponseTablesItem",
    "RankingsResponseTablesPropertiesItemsItem",
    "RankingsResponseTablesPropertiesItemsMetricsItem",
    "RateLimitRule",
    "RateLimitRuleList",
    "RemoteAddressMatchConditionParameters",
    "RequestBodyMatchConditionParameters",
    "RequestHeaderMatchConditionParameters",
    "RequestMethodMatchConditionParameters",
    "RequestSchemeMatchConditionParameters",
    "RequestUriMatchConditionParameters",
    "Resource",
    "ResourceReference",
    "ResourceUsage",
    "ResourceUsageListResult",
    "ResourcesResponse",
    "ResourcesResponseCustomDomainsItem",
    "ResourcesResponseEndpointsItem",
    "ResourcesResponseEndpointsPropertiesItemsItem",
    "ResponseBasedOriginErrorDetectionParameters",
    "Route",
    "RouteConfigurationOverrideActionParameters",
    "RouteListResult",
    "RouteProperties",
    "RouteUpdateParameters",
    "RouteUpdatePropertiesParameters",
    "Rule",
    "RuleListResult",
    "RuleProperties",
    "RuleSet",
    "RuleSetListResult",
    "RuleSetProperties",
    "RuleUpdateParameters",
    "RuleUpdatePropertiesParameters",
    "Secret",
    "SecretListResult",
    "SecretParameters",
    "SecretProperties",
    "SecurityPolicy",
    "SecurityPolicyListResult",
    "SecurityPolicyProperties",
    "SecurityPolicyPropertiesParameters",
    "SecurityPolicyUpdateParameters",
    "SecurityPolicyWebApplicationFirewallAssociation",
    "SecurityPolicyWebApplicationFirewallParameters",
    "ServerPortMatchConditionParameters",
    "ServiceSpecification",
    "SharedPrivateLinkResourceProperties",
    "Sku",
    "SocketAddrMatchConditionParameters",
    "SslProtocolMatchConditionParameters",
    "SsoUri",
    "SupportedOptimizationTypesListResult",
    "SystemData",
    "TrackedResource",
    "UrlFileExtensionMatchConditionParameters",
    "UrlFileNameMatchConditionParameters",
    "UrlPathMatchConditionParameters",
    "UrlRedirectAction",
    "UrlRedirectActionParameters",
    "UrlRewriteAction",
    "UrlRewriteActionParameters",
    "UrlSigningAction",
    "UrlSigningActionParameters",
    "UrlSigningKey",
    "UrlSigningKeyParameters",
    "UrlSigningParamIdentifier",
    "Usage",
    "UsageName",
    "UsagesListResult",
    "UserAssignedIdentity",
    "UserManagedHttpsParameters",
    "ValidateCustomDomainInput",
    "ValidateCustomDomainOutput",
    "ValidateProbeInput",
    "ValidateProbeOutput",
    "ValidateSecretInput",
    "ValidateSecretOutput",
    "ValidationToken",
    "WafMetricsResponse",
    "WafMetricsResponseSeriesItem",
    "WafMetricsResponseSeriesPropertiesItemsItem",
    "WafRankingsResponse",
    "WafRankingsResponseDataItem",
    "AFDEndpointProtocols",
    "ActionType",
    "AfdCertificateType",
    "AfdMinimumTlsVersion",
    "AfdProvisioningState",
    "AfdQueryStringCachingBehavior",
    "Algorithm",
    "AutoGeneratedDomainNameLabelScope",
    "CacheBehavior",
    "CacheExpirationActionParametersTypeName",
    "CacheKeyQueryStringActionParametersTypeName",
    "CacheType",
    "CanMigrateDefaultSku",
    "CdnCertificateSourceParametersTypeName",
    "CertificateSource",
    "CertificateType",
    "ClientPortMatchConditionParametersTypeName",
    "ClientPortOperator",
    "CookiesMatchConditionParametersTypeName",
    "CookiesOperator",
    "CustomDomainResourceState",
    "CustomHttpsProvisioningState",
    "CustomHttpsProvisioningSubstate",
    "CustomRuleEnabledState",
    "DeleteRule",
    "DeliveryRuleActionEnum",
    "DeploymentStatus",
    "DestinationProtocol",
    "DomainValidationState",
    "EnabledState",
    "EndpointProvisioningState",
    "EndpointResourceState",
    "ForwardingProtocol",
    "GeoFilterActions",
    "HeaderAction",
    "HeaderActionParametersTypeName",
    "HealthProbeRequestType",
    "HostNameMatchConditionParametersTypeName",
    "HostNameOperator",
    "HttpVersionMatchConditionParametersTypeName",
    "HttpVersionOperator",
    "HttpsRedirect",
    "IdentityType",
    "IsDeviceMatchConditionParametersMatchValuesItem",
    "IsDeviceMatchConditionParametersTypeName",
    "IsDeviceOperator",
    "KeyVaultCertificateSourceParametersTypeName",
    "KeyVaultSigningKeyParametersTypeName",
    "LinkToDefaultDomain",
    "LogMetric",
    "LogMetricsGranularity",
    "LogMetricsGroupBy",
    "LogRanking",
    "LogRankingMetric",
    "ManagedRuleEnabledState",
    "ManagedServiceIdentityType",
    "MatchProcessingBehavior",
    "MatchVariable",
    "MetricsGranularity",
    "MetricsSeriesUnit",
    "MinimumTlsVersion",
    "Operator",
    "OptimizationType",
    "OriginGroupOverrideActionParametersTypeName",
    "OriginGroupProvisioningState",
    "OriginGroupResourceState",
    "OriginProvisioningState",
    "OriginResourceState",
    "ParamIndicator",
    "PolicyEnabledState",
    "PolicyMode",
    "PolicyResourceState",
    "PolicySettingsDefaultCustomBlockResponseStatusCode",
    "PostArgsMatchConditionParametersTypeName",
    "PostArgsOperator",
    "PrivateEndpointStatus",
    "ProbeProtocol",
    "ProfileProvisioningState",
    "ProfileResourceState",
    "ProfileScrubbingState",
    "ProtocolType",
    "ProvisioningState",
    "QueryStringBehavior",
    "QueryStringCachingBehavior",
    "QueryStringMatchConditionParametersTypeName",
    "QueryStringOperator",
    "RedirectType",
    "RemoteAddressMatchConditionParametersTypeName",
    "RemoteAddressOperator",
    "RequestBodyMatchConditionParametersTypeName",
    "RequestBodyOperator",
    "RequestHeaderMatchConditionParametersTypeName",
    "RequestHeaderOperator",
    "RequestMethodMatchConditionParametersMatchValuesItem",
    "RequestMethodMatchConditionParametersTypeName",
    "RequestMethodOperator",
    "RequestSchemeMatchConditionParametersMatchValuesItem",
    "RequestSchemeMatchConditionParametersOperator",
    "RequestSchemeMatchConditionParametersTypeName",
    "RequestUriMatchConditionParametersTypeName",
    "RequestUriOperator",
    "ResourceType",
    "ResourceUsageUnit",
    "ResponseBasedDetectedErrorTypes",
    "RouteConfigurationOverrideActionParametersTypeName",
    "RuleCacheBehavior",
    "RuleIsCompressionEnabled",
    "RuleQueryStringCachingBehavior",
    "ScrubbingRuleEntryMatchOperator",
    "ScrubbingRuleEntryMatchVariable",
    "ScrubbingRuleEntryState",
    "SecretType",
    "SecurityPolicyType",
    "ServerPortMatchConditionParametersTypeName",
    "ServerPortOperator",
    "SharedPrivateLinkResourceStatus",
    "SkuName",
    "SocketAddrMatchConditionParametersTypeName",
    "SocketAddrOperator",
    "SslProtocol",
    "SslProtocolMatchConditionParametersTypeName",
    "SslProtocolOperator",
    "Status",
    "Transform",
    "TransformType",
    "UpdateRule",
    "UrlFileExtensionMatchConditionParametersTypeName",
    "UrlFileExtensionOperator",
    "UrlFileNameMatchConditionParametersTypeName",
    "UrlFileNameOperator",
    "UrlPathMatchConditionParametersTypeName",
    "UrlPathOperator",
    "UrlRedirectActionParametersTypeName",
    "UrlRewriteActionParametersTypeName",
    "UrlSigningActionParametersTypeName",
    "UsageUnit",
    "WafAction",
    "WafGranularity",
    "WafMatchVariable",
    "WafMetric",
    "WafMetricsGranularity",
    "WafMetricsSeriesUnit",
    "WafRankingGroupBy",
    "WafRankingType",
    "WafRuleType",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
