# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.billing.aio import BillingManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestBillingManagementAddressOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(BillingManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_validate(self, resource_group):
        response = await self.client.address.validate(
            parameters={
                "addressLine1": "str",
                "country": "str",
                "addressLine2": "str",
                "addressLine3": "str",
                "city": "str",
                "companyName": "str",
                "district": "str",
                "email": "str",
                "firstName": "str",
                "isValidAddress": bool,
                "lastName": "str",
                "middleName": "str",
                "phoneNumber": "str",
                "postalCode": "str",
                "region": "str",
            },
            api_version="2024-04-01",
        )

        # please add some check logic here by yourself
        # ...
