# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import ClientDefaultValueClientTestBase, DefaultValuePreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDefaultValue(ClientDefaultValueClientTestBase):
    @DefaultValuePreparer()
    @recorded_by_proxy
    def test_put_model_property(self, defaultvalue_endpoint):
        client = self.create_client(endpoint=defaultvalue_endpoint)
        response = client.put_model_property(
            body={"name": "str", "retry": bool, "tier": "str", "timeout": 0},
        )

        # please add some check logic here by yourself
        # ...

    @DefaultValuePreparer()
    @recorded_by_proxy
    def test_get_operation_parameter(self, defaultvalue_endpoint):
        client = self.create_client(endpoint=defaultvalue_endpoint)
        response = client.get_operation_parameter(
            name="str",
        )

        # please add some check logic here by yourself
        # ...

    @DefaultValuePreparer()
    @recorded_by_proxy
    def test_get_path_parameter(self, defaultvalue_endpoint):
        client = self.create_client(endpoint=defaultvalue_endpoint)
        response = client.get_path_parameter(
            segment2="str",
            segment1="str",
        )

        # please add some check logic here by yourself
        # ...

    @DefaultValuePreparer()
    @recorded_by_proxy
    def test_get_header_parameter(self, defaultvalue_endpoint):
        client = self.create_client(endpoint=defaultvalue_endpoint)
        response = client.get_header_parameter()

        # please add some check logic here by yourself
        # ...
