# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.billingbenefits import BillingBenefitsMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-billingbenefits
# USAGE
    python savings_plan_order_alias_create_single_scope.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BillingBenefitsMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.savings_plan_order_alias.begin_create(
        savings_plan_order_alias_name="spAlias123",
        body={
            "properties": {
                "appliedScopeProperties": {"subscriptionId": "/subscriptions/30000000-0000-0000-0000-000000000000"},
                "appliedScopeType": "Single",
                "billingPlan": "P1M",
                "billingScopeId": "/providers/Microsoft.Billing/billingAccounts/1234567/billingSubscriptions/30000000-0000-0000-0000-000000000000",
                "commitment": {"amount": 0.001, "currencyCode": "USD", "grain": "Hourly"},
                "displayName": "Compute_SavingsPlan_10-28-2022_16-38",
                "term": "P3Y",
            },
            "sku": {"name": "Compute_Savings_Plan"},
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-12-01-preview/SavingsPlanOrderAliasCreateSingleScope.json
if __name__ == "__main__":
    main()
