# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._cdn_management_client_operations import CdnManagementClientOperationsMixin
from ._afd_profiles_operations import AFDProfilesOperations
from ._afd_custom_domains_operations import AFDCustomDomainsOperations
from ._afd_endpoints_operations import AFDEndpointsOperations
from ._afd_origin_groups_operations import AFDOriginGroupsOperations
from ._afd_origins_operations import AFDOriginsOperations
from ._routes_operations import RoutesOperations
from ._rule_sets_operations import RuleSetsOperations
from ._rules_operations import RulesOperations
from ._security_policies_operations import SecurityPoliciesOperations
from ._secrets_operations import SecretsOperations
from ._log_analytics_operations import LogAnalyticsOperations
from ._profiles_operations import ProfilesOperations
from ._endpoints_operations import EndpointsOperations
from ._origins_operations import OriginsOperations
from ._origin_groups_operations import OriginGroupsOperations
from ._custom_domains_operations import CustomDomainsOperations
from ._resource_usage_operations import ResourceUsageOperations
from ._operations import Operations
from ._edge_nodes_operations import EdgeNodesOperations
from ._policies_operations import PoliciesOperations
from ._managed_rule_sets_operations import ManagedRuleSetsOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "CdnManagementClientOperationsMixin",
    "AFDProfilesOperations",
    "AFDCustomDomainsOperations",
    "AFDEndpointsOperations",
    "AFDOriginGroupsOperations",
    "AFDOriginsOperations",
    "RoutesOperations",
    "RuleSetsOperations",
    "RulesOperations",
    "SecurityPoliciesOperations",
    "SecretsOperations",
    "LogAnalyticsOperations",
    "ProfilesOperations",
    "EndpointsOperations",
    "OriginsOperations",
    "OriginGroupsOperations",
    "CustomDomainsOperations",
    "ResourceUsageOperations",
    "Operations",
    "EdgeNodesOperations",
    "PoliciesOperations",
    "ManagedRuleSetsOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
