# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._operations import ApplicationDataOperations
from ._operations import AttachmentsOperations
from ._operations import BoundariesOperations
from ._operations import CropProductsOperations
from ._operations import CropsOperations
from ._operations import DeviceDataModelsOperations
from ._operations import DevicesOperations
from ._operations import FarmOperationsOperations
from ._operations import FarmsOperations
from ._operations import FieldsOperations
from ._operations import HarvestDataOperations
from ._operations import ImageProcessingOperations
from ._operations import InsightAttachmentsOperations
from ._operations import InsightsOperations
from ._operations import ManagementZonesOperations
from ._operations import ModelInferenceOperations
from ._operations import NutrientAnalysesOperations
from ._operations import OAuthProvidersOperations
from ._operations import OAuthTokensOperations
from ._operations import PartiesOperations
from ._operations import PlantingDataOperations
from ._operations import PlantTissueAnalysesOperations
from ._operations import PrescriptionMapsOperations
from ._operations import PrescriptionsOperations
from ._operations import ScenesOperations
from ._operations import SeasonalFieldsOperations
from ._operations import SeasonsOperations
from ._operations import SensorDataModelsOperations
from ._operations import SensorEventsOperations
from ._operations import SensorMappingsOperations
from ._operations import SensorPartnerIntegrationsOperations
from ._operations import SensorsOperations
from ._operations import SolutionInferenceOperations
from ._operations import TillageDataOperations
from ._operations import WeatherOperations
from ._operations import WeatherDataOperations
from ._operations import ZonesOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # type: ignore # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ApplicationDataOperations",
    "AttachmentsOperations",
    "BoundariesOperations",
    "CropProductsOperations",
    "CropsOperations",
    "DeviceDataModelsOperations",
    "DevicesOperations",
    "FarmOperationsOperations",
    "FarmsOperations",
    "FieldsOperations",
    "HarvestDataOperations",
    "ImageProcessingOperations",
    "InsightAttachmentsOperations",
    "InsightsOperations",
    "ManagementZonesOperations",
    "ModelInferenceOperations",
    "NutrientAnalysesOperations",
    "OAuthProvidersOperations",
    "OAuthTokensOperations",
    "PartiesOperations",
    "PlantingDataOperations",
    "PlantTissueAnalysesOperations",
    "PrescriptionMapsOperations",
    "PrescriptionsOperations",
    "ScenesOperations",
    "SeasonalFieldsOperations",
    "SeasonsOperations",
    "SensorDataModelsOperations",
    "SensorEventsOperations",
    "SensorMappingsOperations",
    "SensorPartnerIntegrationsOperations",
    "SensorsOperations",
    "SolutionInferenceOperations",
    "TillageDataOperations",
    "WeatherOperations",
    "WeatherDataOperations",
    "ZonesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
