# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import ActivationKeyResult
from ._models_py3 import CloudManifestFileDeploymentData
from ._models_py3 import CloudManifestFileProperties
from ._models_py3 import CloudManifestFileResponse
from ._models_py3 import Compatibility
from ._models_py3 import CustomerSubscription
from ._models_py3 import CustomerSubscriptionList
from ._models_py3 import DataDiskImage
from ._models_py3 import DeploymentLicenseRequest
from ._models_py3 import DeploymentLicenseResponse
from ._models_py3 import DeviceConfiguration
from ._models_py3 import Display
from ._models_py3 import ErrorDetails
from ._models_py3 import ErrorResponse
from ._models_py3 import ExtendedProduct
from ._models_py3 import ExtendedProductProperties
from ._models_py3 import IconUris
from ._models_py3 import MarketplaceProductLogUpdate
from ._models_py3 import Operation
from ._models_py3 import OperationList
from ._models_py3 import OsDiskImage
from ._models_py3 import Product
from ._models_py3 import ProductLink
from ._models_py3 import ProductList
from ._models_py3 import ProductLog
from ._models_py3 import ProductProperties
from ._models_py3 import Registration
from ._models_py3 import RegistrationList
from ._models_py3 import RegistrationParameter
from ._models_py3 import Resource
from ._models_py3 import TrackedResource
from ._models_py3 import VirtualMachineExtensionProductProperties
from ._models_py3 import VirtualMachineProductProperties

from ._azure_stack_management_client_enums import Category
from ._azure_stack_management_client_enums import CompatibilityIssue
from ._azure_stack_management_client_enums import ComputeRole
from ._azure_stack_management_client_enums import Location
from ._azure_stack_management_client_enums import OperatingSystem
from ._azure_stack_management_client_enums import ProvisioningState
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "ActivationKeyResult",
    "CloudManifestFileDeploymentData",
    "CloudManifestFileProperties",
    "CloudManifestFileResponse",
    "Compatibility",
    "CustomerSubscription",
    "CustomerSubscriptionList",
    "DataDiskImage",
    "DeploymentLicenseRequest",
    "DeploymentLicenseResponse",
    "DeviceConfiguration",
    "Display",
    "ErrorDetails",
    "ErrorResponse",
    "ExtendedProduct",
    "ExtendedProductProperties",
    "IconUris",
    "MarketplaceProductLogUpdate",
    "Operation",
    "OperationList",
    "OsDiskImage",
    "Product",
    "ProductLink",
    "ProductList",
    "ProductLog",
    "ProductProperties",
    "Registration",
    "RegistrationList",
    "RegistrationParameter",
    "Resource",
    "TrackedResource",
    "VirtualMachineExtensionProductProperties",
    "VirtualMachineProductProperties",
    "Category",
    "CompatibilityIssue",
    "ComputeRole",
    "Location",
    "OperatingSystem",
    "ProvisioningState",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
