# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._operations import DiscountsOperations  # type: ignore
from ._operations import ContributorsOperations  # type: ignore
from ._operations import CreditsOperations  # type: ignore
from ._operations import ConditionalCreditsOperations  # type: ignore
from ._operations import ConditionalCreditContributorsOperations  # type: ignore
from ._operations import MaccsOperations  # type: ignore
from ._operations import SavingsPlanOrderAliasOperations  # type: ignore
from ._operations import SavingsPlanOrderOperations  # type: ignore
from ._operations import SavingsPlanOperations  # type: ignore
from ._operations import ReservationOrderAliasOperations  # type: ignore
from ._operations import DiscountOperations  # type: ignore
from ._operations import BenefitOperations  # type: ignore
from ._operations import SellerResourceOperations  # type: ignore
from ._operations import ApplicableMaccsOperations  # type: ignore
from ._operations import SourcesOperations  # type: ignore
from ._operations import FreeServicesOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "DiscountsOperations",
    "ContributorsOperations",
    "CreditsOperations",
    "ConditionalCreditsOperations",
    "ConditionalCreditContributorsOperations",
    "MaccsOperations",
    "SavingsPlanOrderAliasOperations",
    "SavingsPlanOrderOperations",
    "SavingsPlanOperations",
    "ReservationOrderAliasOperations",
    "DiscountOperations",
    "BenefitOperations",
    "SellerResourceOperations",
    "ApplicableMaccsOperations",
    "SourcesOperations",
    "FreeServicesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
