# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.appcontainers import ContainerAppsAPIClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestContainerAppsAPIBuildsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ContainerAppsAPIClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_builds_get(self, resource_group):
        response = self.client.builds.get(
            resource_group_name=resource_group.name,
            builder_name="str",
            build_name="str",
            api_version="2025-02-02-preview",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_builds_begin_create_or_update(self, resource_group):
        response = self.client.builds.begin_create_or_update(
            resource_group_name=resource_group.name,
            builder_name="str",
            build_name="str",
            build_envelope={
                "buildStatus": "str",
                "configuration": {
                    "baseOs": "str",
                    "environmentVariables": [{"name": "str", "value": "str"}],
                    "platform": "str",
                    "platformVersion": "str",
                    "preBuildSteps": [
                        {
                            "description": "str",
                            "httpGet": {"url": "str", "fileName": "str", "headers": ["str"]},
                            "scripts": ["str"],
                        }
                    ],
                },
                "destinationContainerRegistry": {"server": "str", "image": "str"},
                "id": "str",
                "logStreamEndpoint": "str",
                "name": "str",
                "provisioningState": "str",
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tokenEndpoint": "str",
                "type": "str",
                "uploadEndpoint": "str",
            },
            api_version="2025-02-02-preview",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_builds_begin_delete(self, resource_group):
        response = self.client.builds.begin_delete(
            resource_group_name=resource_group.name,
            builder_name="str",
            build_name="str",
            api_version="2025-02-02-preview",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...
