# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils import recorded_by_proxy
from testpreparer import HierarchyBuildingClientTestBase, HierarchyBuildingPreparer


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestHierarchyBuildingAnimalOperationsOperations(HierarchyBuildingClientTestBase):
    @HierarchyBuildingPreparer()
    @recorded_by_proxy
    def test_animal_operations_update_pet_as_animal(self, hierarchybuilding_endpoint):
        client = self.create_client(endpoint=hierarchybuilding_endpoint)
        response = client.animal_operations.update_pet_as_animal(
            animal={"breed": "str", "kind": "dog", "name": "str", "trained": bool},
        )

        # please add some check logic here by yourself
        # ...

    @HierarchyBuildingPreparer()
    @recorded_by_proxy
    def test_animal_operations_update_dog_as_animal(self, hierarchybuilding_endpoint):
        client = self.create_client(endpoint=hierarchybuilding_endpoint)
        response = client.animal_operations.update_dog_as_animal(
            animal={"breed": "str", "kind": "dog", "name": "str", "trained": bool},
        )

        # please add some check logic here by yourself
        # ...
