# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from .._utils.model_base import Model as _Model


class EmptyInput(_Model):
    """Empty model used in operation parameters."""


class EmptyInputOutput(_Model):
    """Empty model used in both parameter and return type."""


class EmptyOutput(_Model):
    """Empty model used in operation return type."""
