# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import AcceleratorAuthSetting
from ._models_py3 import AcceleratorBasicAuthSetting
from ._models_py3 import AcceleratorGitRepository
from ._models_py3 import AcceleratorPublicSetting
from ._models_py3 import AcceleratorSshSetting
from ._models_py3 import ActiveDeploymentCollection
from ._models_py3 import ApiPortalCustomDomainProperties
from ._models_py3 import ApiPortalCustomDomainResource
from ._models_py3 import ApiPortalCustomDomainResourceCollection
from ._models_py3 import ApiPortalInstance
from ._models_py3 import ApiPortalProperties
from ._models_py3 import ApiPortalResource
from ._models_py3 import ApiPortalResourceCollection
from ._models_py3 import ApiPortalResourceRequests
from ._models_py3 import ApmProperties
from ._models_py3 import ApmReference
from ._models_py3 import ApmResource
from ._models_py3 import ApmResourceCollection
from ._models_py3 import ApmSecretKeys
from ._models_py3 import AppResource
from ._models_py3 import AppResourceCollection
from ._models_py3 import AppResourceProperties
from ._models_py3 import AppVNetAddons
from ._models_py3 import ApplicationAcceleratorComponent
from ._models_py3 import ApplicationAcceleratorInstance
from ._models_py3 import ApplicationAcceleratorProperties
from ._models_py3 import ApplicationAcceleratorResource
from ._models_py3 import ApplicationAcceleratorResourceCollection
from ._models_py3 import ApplicationAcceleratorResourceRequests
from ._models_py3 import ApplicationInsightsAgentVersions
from ._models_py3 import ApplicationLiveViewComponent
from ._models_py3 import ApplicationLiveViewInstance
from ._models_py3 import ApplicationLiveViewProperties
from ._models_py3 import ApplicationLiveViewResource
from ._models_py3 import ApplicationLiveViewResourceCollection
from ._models_py3 import ApplicationLiveViewResourceRequests
from ._models_py3 import AvailableOperations
from ._models_py3 import AvailableRuntimeVersions
from ._models_py3 import AzureFileVolume
from ._models_py3 import BindingResource
from ._models_py3 import BindingResourceCollection
from ._models_py3 import BindingResourceProperties
from ._models_py3 import Build
from ._models_py3 import BuildCollection
from ._models_py3 import BuildProperties
from ._models_py3 import BuildResourceRequests
from ._models_py3 import BuildResult
from ._models_py3 import BuildResultCollection
from ._models_py3 import BuildResultLog
from ._models_py3 import BuildResultProperties
from ._models_py3 import BuildResultUserSourceInfo
from ._models_py3 import BuildService
from ._models_py3 import BuildServiceAgentPoolProperties
from ._models_py3 import BuildServiceAgentPoolResource
from ._models_py3 import BuildServiceAgentPoolResourceCollection
from ._models_py3 import BuildServiceAgentPoolSizeProperties
from ._models_py3 import BuildServiceCollection
from ._models_py3 import BuildServiceProperties
from ._models_py3 import BuildServicePropertiesResourceRequests
from ._models_py3 import BuildStageProperties
from ._models_py3 import BuilderProperties
from ._models_py3 import BuilderResource
from ._models_py3 import BuilderResourceCollection
from ._models_py3 import BuildpackBindingLaunchProperties
from ._models_py3 import BuildpackBindingProperties
from ._models_py3 import BuildpackBindingResource
from ._models_py3 import BuildpackBindingResourceCollection
from ._models_py3 import BuildpackProperties
from ._models_py3 import BuildpacksGroupProperties
from ._models_py3 import CertificateProperties
from ._models_py3 import CertificateReference
from ._models_py3 import CertificateResource
from ._models_py3 import CertificateResourceCollection
from ._models_py3 import CloudErrorBody
from ._models_py3 import ClusterResourceProperties
from ._models_py3 import ConfigServerGitProperty
from ._models_py3 import ConfigServerProperties
from ._models_py3 import ConfigServerResource
from ._models_py3 import ConfigServerSettings
from ._models_py3 import ConfigServerSettingsErrorRecord
from ._models_py3 import ConfigServerSettingsValidateResult
from ._models_py3 import ConfigurationServiceGitProperty
from ._models_py3 import ConfigurationServiceGitPropertyValidateResult
from ._models_py3 import ConfigurationServiceGitRepository
from ._models_py3 import ConfigurationServiceInstance
from ._models_py3 import ConfigurationServiceProperties
from ._models_py3 import ConfigurationServiceResource
from ._models_py3 import ConfigurationServiceResourceCollection
from ._models_py3 import ConfigurationServiceResourceRequests
from ._models_py3 import ConfigurationServiceSettings
from ._models_py3 import ConfigurationServiceSettingsValidateResult
from ._models_py3 import ContainerProbeSettings
from ._models_py3 import ContainerRegistryBasicCredentials
from ._models_py3 import ContainerRegistryCredentials
from ._models_py3 import ContainerRegistryProperties
from ._models_py3 import ContainerRegistryResource
from ._models_py3 import ContainerRegistryResourceCollection
from ._models_py3 import ContainerRegistryValidateResult
from ._models_py3 import ContentCertificateProperties
from ._models_py3 import CustomContainer
from ._models_py3 import CustomContainerUserSourceInfo
from ._models_py3 import CustomDomainProperties
from ._models_py3 import CustomDomainResource
from ._models_py3 import CustomDomainResourceCollection
from ._models_py3 import CustomDomainValidatePayload
from ._models_py3 import CustomDomainValidateResult
from ._models_py3 import CustomPersistentDiskProperties
from ._models_py3 import CustomPersistentDiskResource
from ._models_py3 import CustomizedAcceleratorProperties
from ._models_py3 import CustomizedAcceleratorResource
from ._models_py3 import CustomizedAcceleratorResourceCollection
from ._models_py3 import CustomizedAcceleratorValidateResult
from ._models_py3 import DeploymentInstance
from ._models_py3 import DeploymentList
from ._models_py3 import DeploymentResource
from ._models_py3 import DeploymentResourceCollection
from ._models_py3 import DeploymentResourceProperties
from ._models_py3 import DeploymentSettings
from ._models_py3 import DevToolPortalComponent
from ._models_py3 import DevToolPortalFeatureDetail
from ._models_py3 import DevToolPortalFeatureSettings
from ._models_py3 import DevToolPortalInstance
from ._models_py3 import DevToolPortalProperties
from ._models_py3 import DevToolPortalResource
from ._models_py3 import DevToolPortalResourceCollection
from ._models_py3 import DevToolPortalResourceRequests
from ._models_py3 import DevToolPortalSsoProperties
from ._models_py3 import DiagnosticParameters
from ._models_py3 import Error
from ._models_py3 import ExecAction
from ._models_py3 import GatewayApiMetadataProperties
from ._models_py3 import GatewayApiRoute
from ._models_py3 import GatewayCorsProperties
from ._models_py3 import GatewayCustomDomainProperties
from ._models_py3 import GatewayCustomDomainResource
from ._models_py3 import GatewayCustomDomainResourceCollection
from ._models_py3 import GatewayInstance
from ._models_py3 import GatewayOperatorProperties
from ._models_py3 import GatewayOperatorResourceRequests
from ._models_py3 import GatewayProperties
from ._models_py3 import GatewayPropertiesClientAuth
from ._models_py3 import GatewayPropertiesEnvironmentVariables
from ._models_py3 import GatewayResource
from ._models_py3 import GatewayResourceCollection
from ._models_py3 import GatewayResourceRequests
from ._models_py3 import GatewayRouteConfigOpenApiProperties
from ._models_py3 import GatewayRouteConfigProperties
from ._models_py3 import GatewayRouteConfigResource
from ._models_py3 import GatewayRouteConfigResourceCollection
from ._models_py3 import GitPatternRepository
from ._models_py3 import GloballyEnabledApms
from ._models_py3 import HTTPGetAction
from ._models_py3 import ImageRegistryCredential
from ._models_py3 import IngressConfig
from ._models_py3 import IngressSettings
from ._models_py3 import IngressSettingsClientAuth
from ._models_py3 import JarUploadedUserSourceInfo
from ._models_py3 import KeyVaultCertificateProperties
from ._models_py3 import LoadedCertificate
from ._models_py3 import LogFileUrlResponse
from ._models_py3 import LogSpecification
from ._models_py3 import ManagedIdentityProperties
from ._models_py3 import MarketplaceResource
from ._models_py3 import MetricDimension
from ._models_py3 import MetricSpecification
from ._models_py3 import MonitoringSettingProperties
from ._models_py3 import MonitoringSettingResource
from ._models_py3 import NameAvailability
from ._models_py3 import NameAvailabilityParameters
from ._models_py3 import NetCoreZipUploadedUserSourceInfo
from ._models_py3 import NetworkProfile
from ._models_py3 import NetworkProfileOutboundIPs
from ._models_py3 import OperationDetail
from ._models_py3 import OperationDisplay
from ._models_py3 import OperationProperties
from ._models_py3 import PersistentDisk
from ._models_py3 import PredefinedAcceleratorProperties
from ._models_py3 import PredefinedAcceleratorResource
from ._models_py3 import PredefinedAcceleratorResourceCollection
from ._models_py3 import Probe
from ._models_py3 import ProbeAction
from ._models_py3 import ProxyResource
from ._models_py3 import RegenerateTestKeyRequestPayload
from ._models_py3 import RemoteDebugging
from ._models_py3 import RemoteDebuggingPayload
from ._models_py3 import RequiredTraffic
from ._models_py3 import Resource
from ._models_py3 import ResourceRequests
from ._models_py3 import ResourceSku
from ._models_py3 import ResourceSkuCapabilities
from ._models_py3 import ResourceSkuCollection
from ._models_py3 import ResourceSkuLocationInfo
from ._models_py3 import ResourceSkuRestrictionInfo
from ._models_py3 import ResourceSkuRestrictions
from ._models_py3 import ResourceSkuZoneDetails
from ._models_py3 import ResourceUploadDefinition
from ._models_py3 import ServiceRegistryInstance
from ._models_py3 import ServiceRegistryProperties
from ._models_py3 import ServiceRegistryResource
from ._models_py3 import ServiceRegistryResourceCollection
from ._models_py3 import ServiceRegistryResourceRequests
from ._models_py3 import ServiceResource
from ._models_py3 import ServiceResourceList
from ._models_py3 import ServiceSpecification
from ._models_py3 import ServiceVNetAddons
from ._models_py3 import Sku
from ._models_py3 import SkuCapacity
from ._models_py3 import SkuObject
from ._models_py3 import SourceUploadedUserSourceInfo
from ._models_py3 import SsoProperties
from ._models_py3 import StackProperties
from ._models_py3 import StorageAccount
from ._models_py3 import StorageProperties
from ._models_py3 import StorageResource
from ._models_py3 import StorageResourceCollection
from ._models_py3 import SupportedApmType
from ._models_py3 import SupportedApmTypes
from ._models_py3 import SupportedBuildpackResource
from ._models_py3 import SupportedBuildpackResourceProperties
from ._models_py3 import SupportedBuildpacksCollection
from ._models_py3 import SupportedRuntimeVersion
from ._models_py3 import SupportedServerVersion
from ._models_py3 import SupportedServerVersions
from ._models_py3 import SupportedStackResource
from ._models_py3 import SupportedStackResourceProperties
from ._models_py3 import SupportedStacksCollection
from ._models_py3 import SystemData
from ._models_py3 import TCPSocketAction
from ._models_py3 import TemporaryDisk
from ._models_py3 import TestKeys
from ._models_py3 import TrackedResource
from ._models_py3 import TriggeredBuildResult
from ._models_py3 import UploadedUserSourceInfo
from ._models_py3 import UserAssignedManagedIdentity
from ._models_py3 import UserSourceInfo
from ._models_py3 import ValidationMessages
from ._models_py3 import WarUploadedUserSourceInfo

from ._app_platform_management_client_enums import ActionType
from ._app_platform_management_client_enums import ApiPortalApiTryOutEnabledState
from ._app_platform_management_client_enums import ApiPortalProvisioningState
from ._app_platform_management_client_enums import ApmProvisioningState
from ._app_platform_management_client_enums import AppResourceProvisioningState
from ._app_platform_management_client_enums import ApplicationAcceleratorProvisioningState
from ._app_platform_management_client_enums import ApplicationLiveViewProvisioningState
from ._app_platform_management_client_enums import BackendProtocol
from ._app_platform_management_client_enums import BindingType
from ._app_platform_management_client_enums import BuildProvisioningState
from ._app_platform_management_client_enums import BuildResultProvisioningState
from ._app_platform_management_client_enums import BuildServiceProvisioningState
from ._app_platform_management_client_enums import BuilderProvisioningState
from ._app_platform_management_client_enums import BuildpackBindingProvisioningState
from ._app_platform_management_client_enums import CertificateResourceProvisioningState
from ._app_platform_management_client_enums import ConfigServerState
from ._app_platform_management_client_enums import ConfigurationServiceGeneration
from ._app_platform_management_client_enums import ConfigurationServiceProvisioningState
from ._app_platform_management_client_enums import ContainerRegistryProvisioningState
from ._app_platform_management_client_enums import CreatedByType
from ._app_platform_management_client_enums import CustomDomainResourceProvisioningState
from ._app_platform_management_client_enums import CustomizedAcceleratorProvisioningState
from ._app_platform_management_client_enums import CustomizedAcceleratorType
from ._app_platform_management_client_enums import CustomizedAcceleratorValidateResultState
from ._app_platform_management_client_enums import DeploymentResourceProvisioningState
from ._app_platform_management_client_enums import DeploymentResourceStatus
from ._app_platform_management_client_enums import DevToolPortalFeatureState
from ._app_platform_management_client_enums import DevToolPortalProvisioningState
from ._app_platform_management_client_enums import GatewayCertificateVerification
from ._app_platform_management_client_enums import GatewayProvisioningState
from ._app_platform_management_client_enums import GatewayRouteConfigProtocol
from ._app_platform_management_client_enums import GitImplementation
from ._app_platform_management_client_enums import HTTPSchemeType
from ._app_platform_management_client_enums import KPackBuildStageProvisioningState
from ._app_platform_management_client_enums import KeyVaultCertificateAutoSync
from ._app_platform_management_client_enums import LastModifiedByType
from ._app_platform_management_client_enums import ManagedIdentityType
from ._app_platform_management_client_enums import MonitoringSettingState
from ._app_platform_management_client_enums import PowerState
from ._app_platform_management_client_enums import PredefinedAcceleratorProvisioningState
from ._app_platform_management_client_enums import PredefinedAcceleratorState
from ._app_platform_management_client_enums import ProbeActionType
from ._app_platform_management_client_enums import ProvisioningState
from ._app_platform_management_client_enums import ResourceSkuRestrictionsReasonCode
from ._app_platform_management_client_enums import ResourceSkuRestrictionsType
from ._app_platform_management_client_enums import ServiceRegistryProvisioningState
from ._app_platform_management_client_enums import SessionAffinity
from ._app_platform_management_client_enums import SkuScaleType
from ._app_platform_management_client_enums import StorageType
from ._app_platform_management_client_enums import SupportedRuntimePlatform
from ._app_platform_management_client_enums import SupportedRuntimeValue
from ._app_platform_management_client_enums import TestKeyType
from ._app_platform_management_client_enums import TrafficDirection
from ._app_platform_management_client_enums import TriggeredBuildResultProvisioningState
from ._app_platform_management_client_enums import Type
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AcceleratorAuthSetting",
    "AcceleratorBasicAuthSetting",
    "AcceleratorGitRepository",
    "AcceleratorPublicSetting",
    "AcceleratorSshSetting",
    "ActiveDeploymentCollection",
    "ApiPortalCustomDomainProperties",
    "ApiPortalCustomDomainResource",
    "ApiPortalCustomDomainResourceCollection",
    "ApiPortalInstance",
    "ApiPortalProperties",
    "ApiPortalResource",
    "ApiPortalResourceCollection",
    "ApiPortalResourceRequests",
    "ApmProperties",
    "ApmReference",
    "ApmResource",
    "ApmResourceCollection",
    "ApmSecretKeys",
    "AppResource",
    "AppResourceCollection",
    "AppResourceProperties",
    "AppVNetAddons",
    "ApplicationAcceleratorComponent",
    "ApplicationAcceleratorInstance",
    "ApplicationAcceleratorProperties",
    "ApplicationAcceleratorResource",
    "ApplicationAcceleratorResourceCollection",
    "ApplicationAcceleratorResourceRequests",
    "ApplicationInsightsAgentVersions",
    "ApplicationLiveViewComponent",
    "ApplicationLiveViewInstance",
    "ApplicationLiveViewProperties",
    "ApplicationLiveViewResource",
    "ApplicationLiveViewResourceCollection",
    "ApplicationLiveViewResourceRequests",
    "AvailableOperations",
    "AvailableRuntimeVersions",
    "AzureFileVolume",
    "BindingResource",
    "BindingResourceCollection",
    "BindingResourceProperties",
    "Build",
    "BuildCollection",
    "BuildProperties",
    "BuildResourceRequests",
    "BuildResult",
    "BuildResultCollection",
    "BuildResultLog",
    "BuildResultProperties",
    "BuildResultUserSourceInfo",
    "BuildService",
    "BuildServiceAgentPoolProperties",
    "BuildServiceAgentPoolResource",
    "BuildServiceAgentPoolResourceCollection",
    "BuildServiceAgentPoolSizeProperties",
    "BuildServiceCollection",
    "BuildServiceProperties",
    "BuildServicePropertiesResourceRequests",
    "BuildStageProperties",
    "BuilderProperties",
    "BuilderResource",
    "BuilderResourceCollection",
    "BuildpackBindingLaunchProperties",
    "BuildpackBindingProperties",
    "BuildpackBindingResource",
    "BuildpackBindingResourceCollection",
    "BuildpackProperties",
    "BuildpacksGroupProperties",
    "CertificateProperties",
    "CertificateReference",
    "CertificateResource",
    "CertificateResourceCollection",
    "CloudErrorBody",
    "ClusterResourceProperties",
    "ConfigServerGitProperty",
    "ConfigServerProperties",
    "ConfigServerResource",
    "ConfigServerSettings",
    "ConfigServerSettingsErrorRecord",
    "ConfigServerSettingsValidateResult",
    "ConfigurationServiceGitProperty",
    "ConfigurationServiceGitPropertyValidateResult",
    "ConfigurationServiceGitRepository",
    "ConfigurationServiceInstance",
    "ConfigurationServiceProperties",
    "ConfigurationServiceResource",
    "ConfigurationServiceResourceCollection",
    "ConfigurationServiceResourceRequests",
    "ConfigurationServiceSettings",
    "ConfigurationServiceSettingsValidateResult",
    "ContainerProbeSettings",
    "ContainerRegistryBasicCredentials",
    "ContainerRegistryCredentials",
    "ContainerRegistryProperties",
    "ContainerRegistryResource",
    "ContainerRegistryResourceCollection",
    "ContainerRegistryValidateResult",
    "ContentCertificateProperties",
    "CustomContainer",
    "CustomContainerUserSourceInfo",
    "CustomDomainProperties",
    "CustomDomainResource",
    "CustomDomainResourceCollection",
    "CustomDomainValidatePayload",
    "CustomDomainValidateResult",
    "CustomPersistentDiskProperties",
    "CustomPersistentDiskResource",
    "CustomizedAcceleratorProperties",
    "CustomizedAcceleratorResource",
    "CustomizedAcceleratorResourceCollection",
    "CustomizedAcceleratorValidateResult",
    "DeploymentInstance",
    "DeploymentList",
    "DeploymentResource",
    "DeploymentResourceCollection",
    "DeploymentResourceProperties",
    "DeploymentSettings",
    "DevToolPortalComponent",
    "DevToolPortalFeatureDetail",
    "DevToolPortalFeatureSettings",
    "DevToolPortalInstance",
    "DevToolPortalProperties",
    "DevToolPortalResource",
    "DevToolPortalResourceCollection",
    "DevToolPortalResourceRequests",
    "DevToolPortalSsoProperties",
    "DiagnosticParameters",
    "Error",
    "ExecAction",
    "GatewayApiMetadataProperties",
    "GatewayApiRoute",
    "GatewayCorsProperties",
    "GatewayCustomDomainProperties",
    "GatewayCustomDomainResource",
    "GatewayCustomDomainResourceCollection",
    "GatewayInstance",
    "GatewayOperatorProperties",
    "GatewayOperatorResourceRequests",
    "GatewayProperties",
    "GatewayPropertiesClientAuth",
    "GatewayPropertiesEnvironmentVariables",
    "GatewayResource",
    "GatewayResourceCollection",
    "GatewayResourceRequests",
    "GatewayRouteConfigOpenApiProperties",
    "GatewayRouteConfigProperties",
    "GatewayRouteConfigResource",
    "GatewayRouteConfigResourceCollection",
    "GitPatternRepository",
    "GloballyEnabledApms",
    "HTTPGetAction",
    "ImageRegistryCredential",
    "IngressConfig",
    "IngressSettings",
    "IngressSettingsClientAuth",
    "JarUploadedUserSourceInfo",
    "KeyVaultCertificateProperties",
    "LoadedCertificate",
    "LogFileUrlResponse",
    "LogSpecification",
    "ManagedIdentityProperties",
    "MarketplaceResource",
    "MetricDimension",
    "MetricSpecification",
    "MonitoringSettingProperties",
    "MonitoringSettingResource",
    "NameAvailability",
    "NameAvailabilityParameters",
    "NetCoreZipUploadedUserSourceInfo",
    "NetworkProfile",
    "NetworkProfileOutboundIPs",
    "OperationDetail",
    "OperationDisplay",
    "OperationProperties",
    "PersistentDisk",
    "PredefinedAcceleratorProperties",
    "PredefinedAcceleratorResource",
    "PredefinedAcceleratorResourceCollection",
    "Probe",
    "ProbeAction",
    "ProxyResource",
    "RegenerateTestKeyRequestPayload",
    "RemoteDebugging",
    "RemoteDebuggingPayload",
    "RequiredTraffic",
    "Resource",
    "ResourceRequests",
    "ResourceSku",
    "ResourceSkuCapabilities",
    "ResourceSkuCollection",
    "ResourceSkuLocationInfo",
    "ResourceSkuRestrictionInfo",
    "ResourceSkuRestrictions",
    "ResourceSkuZoneDetails",
    "ResourceUploadDefinition",
    "ServiceRegistryInstance",
    "ServiceRegistryProperties",
    "ServiceRegistryResource",
    "ServiceRegistryResourceCollection",
    "ServiceRegistryResourceRequests",
    "ServiceResource",
    "ServiceResourceList",
    "ServiceSpecification",
    "ServiceVNetAddons",
    "Sku",
    "SkuCapacity",
    "SkuObject",
    "SourceUploadedUserSourceInfo",
    "SsoProperties",
    "StackProperties",
    "StorageAccount",
    "StorageProperties",
    "StorageResource",
    "StorageResourceCollection",
    "SupportedApmType",
    "SupportedApmTypes",
    "SupportedBuildpackResource",
    "SupportedBuildpackResourceProperties",
    "SupportedBuildpacksCollection",
    "SupportedRuntimeVersion",
    "SupportedServerVersion",
    "SupportedServerVersions",
    "SupportedStackResource",
    "SupportedStackResourceProperties",
    "SupportedStacksCollection",
    "SystemData",
    "TCPSocketAction",
    "TemporaryDisk",
    "TestKeys",
    "TrackedResource",
    "TriggeredBuildResult",
    "UploadedUserSourceInfo",
    "UserAssignedManagedIdentity",
    "UserSourceInfo",
    "ValidationMessages",
    "WarUploadedUserSourceInfo",
    "ActionType",
    "ApiPortalApiTryOutEnabledState",
    "ApiPortalProvisioningState",
    "ApmProvisioningState",
    "AppResourceProvisioningState",
    "ApplicationAcceleratorProvisioningState",
    "ApplicationLiveViewProvisioningState",
    "BackendProtocol",
    "BindingType",
    "BuildProvisioningState",
    "BuildResultProvisioningState",
    "BuildServiceProvisioningState",
    "BuilderProvisioningState",
    "BuildpackBindingProvisioningState",
    "CertificateResourceProvisioningState",
    "ConfigServerState",
    "ConfigurationServiceGeneration",
    "ConfigurationServiceProvisioningState",
    "ContainerRegistryProvisioningState",
    "CreatedByType",
    "CustomDomainResourceProvisioningState",
    "CustomizedAcceleratorProvisioningState",
    "CustomizedAcceleratorType",
    "CustomizedAcceleratorValidateResultState",
    "DeploymentResourceProvisioningState",
    "DeploymentResourceStatus",
    "DevToolPortalFeatureState",
    "DevToolPortalProvisioningState",
    "GatewayCertificateVerification",
    "GatewayProvisioningState",
    "GatewayRouteConfigProtocol",
    "GitImplementation",
    "HTTPSchemeType",
    "KPackBuildStageProvisioningState",
    "KeyVaultCertificateAutoSync",
    "LastModifiedByType",
    "ManagedIdentityType",
    "MonitoringSettingState",
    "PowerState",
    "PredefinedAcceleratorProvisioningState",
    "PredefinedAcceleratorState",
    "ProbeActionType",
    "ProvisioningState",
    "ResourceSkuRestrictionsReasonCode",
    "ResourceSkuRestrictionsType",
    "ServiceRegistryProvisioningState",
    "SessionAffinity",
    "SkuScaleType",
    "StorageType",
    "SupportedRuntimePlatform",
    "SupportedRuntimeValue",
    "TestKeyType",
    "TrafficDirection",
    "TriggeredBuildResultProvisioningState",
    "Type",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
